# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.sss.v2 import service_menu

CATEGORIES_EXAMPLE = [
    {
        "category_name": "menu.category.test1",
        "sub_categories": [
            {
                "sub_category_name": "menu.subcategory.test1",
                "service_menus": [
                    {
                        "menu_name": "sss.menu.test1",
                        "areas": [
                            {"region_name": "lab3ec"},
                            {"region_name": "lab4ec"},
                            {"region_name": "lab5ec"}
                        ]
                    }
                ]
            }
        ]
    }
]

USER_SERVICE_MENU_EXAMPLE = {
    'user_id': 'user1234567890',
    'categories': CATEGORIES_EXAMPLE
}


class TestServiceMenu(testtools.TestCase):

    def test_service_menu(self):
        sot = service_menu.ServiceMenu()
        self.assertEqual(None, sot.resources_key, message='resouces_key')
        self.assertEqual(None, sot.resource_key, message='resource_key')
        self.assertEqual('', sot.base_path, message='base_path')
        self.assertEqual('sssv2', sot.service.service_type,
                         message='service_type')

        self.assertFalse(sot.allow_create, msg='allow_create')
        self.assertFalse(sot.allow_get, msg='allow_get')
        self.assertFalse(sot.allow_update, msg='allow_update')
        self.assertFalse(sot.allow_delete, msg='allow_delete')
        self.assertFalse(sot.allow_list, msg='allow_list')
        self.assertFalse(sot.allow_head, msg='allow_head')
        self.assertFalse(sot.patch_update, msg='patch_update')
        self.assertFalse(sot.put_create, msg='put_create')

    def test_make_user_service_menu(self):
        sot = service_menu.ServiceMenu(**USER_SERVICE_MENU_EXAMPLE)
        self.assertEqual(USER_SERVICE_MENU_EXAMPLE['user_id'], sot.user_id)
        self.assertEqual(CATEGORIES_EXAMPLE, sot.categories)



from . import base

from ecl import resource2
from ecl.mvna import mvna_service


class HealthMonitor(base.MVNABaseResource):
    resource_key = "health_monitor"
    resources_key = "health_monitors"
    service = mvna_service.MVNAService("v1.0")
    base_path = '/' + service.version + '/health_monitors'

    _query_mapping = base.MVNAQueryParameters(
        "id",
        "name",
        "description",
        "configuration_status",
        "operation_status",
        "port",
        "protocol",
        "interval",
        "retry",
        "timeout",
        "path",
        "http_status_code",
        "load_balancer_id",
        "tenant_id"
    )

    # Capabilities
    allow_list = True
    allow_get = True
    allow_create = True
    allow_update = True
    allow_delete = True
    patch_update = True

    # Properties
    #: It identifies connection resource uniquely
    id = resource2.Body('id')
    #: Name of health monitor
    name = resource2.Body('name')
    #: Description of health monitor
    description = resource2.Body('description')
    #: Tags of health monitor
    tags = resource2.Body('tags')
    #: Configuration status of health monitor
    configuration_status = resource2.Body('configuration_status')
    #: Operation status of health monitor
    operation_status = resource2.Body('operation_status')
    #: Port of health monitor
    port = resource2.Body('port')
    #: Protocol of health monitor
    protocol = resource2.Body('protocol')
    #: Interval of health monitor
    interval = resource2.Body('interval')
    #: Retry count of health monitor
    retry = resource2.Body('retry')
    #: Timeout of health monitor
    timeout = resource2.Body('timeout')

    #: Path of health monitor
    path = resource2.Body('path')
    #: HTTP status code of health monitor
    http_status_code = resource2.Body('http_status_code')

    #: Load balancer ID
    load_balancer_id = resource2.Body('load_balancer_id')
    #: Tenant ID of health monitor
    tenant_id = resource2.Body('tenant_id')
    #: Current configuration of health monitor
    current = resource2.Body('current')
    #: Staged configuration of health monitor
    staged = resource2.Body('staged')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.baremetal import baremetal_service
from ecl import resource2


class Chassis(resource2.Resource):
    """Chassis Resource"""

    resource_key = 'chassis'
    resources_key = 'chassis'
    base_path = '/chassis'
    service = baremetal_service.BaremetalService()

    # Capabilities
    allow_get = True
    allow_list = True

    # Properties
    #: The ID for the chassis, which is a unique integer value.
    id = resource2.Body('id')
    #: The name of availability zone where chassis exists.
    availability_zone = resource2.Body('availability_zone')
    #: The name of flavor of chassis.
    flavor_name = resource2.Body('flavor_name')
    #: The object of summarized hardware spec. That has cpus, disks and rams.
    hardware_summary = resource2.Body('hardware_summary', type=dict)
    #: The status of chassis.
    status = resource2.Body('status')
    #: The ID of server attached to chassis. If no server is attached to chassis, the value is null.
    server_id = resource2.Body('server_id')
    #: The name of server attached to chassis. If no server is attached to chassis, the value is null.
    server_name = resource2.Body('server_name')
    #: The minimum contract period of your chassis.
    contract_year = resource2.Body('contract_year', type=int)
    #: The date that you start to use the chassis.
    start_time = resource2.Body('start_time')

    # Properties (for Detail)
    #: The spec of all cpus installed in chassis.
    cpus = resource2.Body('cpus', type=list)
    #: The spec of all disks installed in chassis.
    disks = resource2.Body('disks', type=list)
    #: The spec of all rams installed in chassis.
    rams = resource2.Body('rams', type=list)


class ChassisDetail(Chassis):
    """ChassisDetail Resource"""

    base_path = '/chassis/detail'

    # Capabilities
    allow_get = False



from . import base

from ecl import resource2
from ecl.mvna import mvna_service


class Route(base.MVNABaseResource):
    resource_key = "route"
    resources_key = "routes"
    service = mvna_service.MVNAService("v1.0")
    base_path = '/' + service.version + '/routes'

    _query_mapping = base.MVNAQueryParameters(
        "id",
        "name",
        "description",
        "configuration_status",
        "operation_status",
        "destination_cidr",
        "next_hop_ip_address",
        "load_balancer_id",
        "tenant_id"
    )

    # Capabilities
    allow_list = True
    allow_get = True
    allow_create = True
    allow_update = True
    allow_delete = True
    patch_update = True

    # Properties
    #: It identifies connection resource uniquely
    id = resource2.Body('id')
    #: Name of route
    name = resource2.Body('name')
    #: Description of route
    description = resource2.Body('description')
    #: Tags of route
    tags = resource2.Body('tags')
    #: Configuration status of route
    configuration_status = resource2.Body('configuration_status')
    #: Operation status of route
    operation_status = resource2.Body('operation_status')
    #: Destination cidr of route
    destination_cidr = resource2.Body('destination_cidr')
    #: Next hop ip address of route
    next_hop_ip_address = resource2.Body('next_hop_ip_address')
    #: Load balancer ID of route
    load_balancer_id = resource2.Body('load_balancer_id')
    #: Tenant ID of route
    tenant_id = resource2.Body('tenant_id')
    #: Current configuration of route
    current = resource2.Body('current')
    #: Staged configuration of route
    staged = resource2.Body('staged')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl import resource2
from ecl.sss import sss_service


class ServiceMenu(resource2.Resource):
    service = sss_service.SssAdminService()

    # Properties
    #: User id. format is ecid[0-9]{10}.
    user_id = resource2.Body('user_id')
    #: Category information of service menu
    categories = resource2.Body('categories')

    def get_user_service_menu(self, session, user_id):
        """Get menu of services available to user.

        :param session: The session to use for making this request.
        :param user_id: ID of a user
        :return: :class:`~ecl.sss.v2.service_menu.ServiceMenu`
        """
        url = '/users/%s/service-menus' % user_id
        resp = session.get(url, endpoint_filter=self.service)
        self._translate_response(resp, has_body=True)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.sss.v2 import token

BASIC_EXAMPLE = {
    "methods": ["password"],
    "user": {
        "domain": {"name": "Default", "id": "default"},
        "name": "USER1",
        "id": "fcb4a033",
        "password_expires_at": None
    },
    "issued_at": "2022-12-12T07:56:28.000000Z",
    "expires_at": "2022-12-12T08:56:28.000000Z",
    "audit_ids": ["cYb6kuPsS1Wiie0pS3m7yg"],
    "project": {
        "domain": {"name": "Default", "id": "default"},
        "id": "32c732c8",
        "name": "test-tenant"
    },
    "is_domain": False,
    "roles": [
        {"id": "db6713f9", "name": "_member_"}
    ],
    "catalog": [
        {
            "endpoints": [
                {"id": "481945da"}
            ]
        }
    ],
    "X-Subject-Token": "token_16718f6557f2",
}


class TestToken(testtools.TestCase):

    def test_token(self):
        sot = token.Token()
        self.assertEqual(None, sot.resources_key)
        self.assertEqual('token', sot.resource_key)
        self.assertEqual('/tokens/users', sot.base_path)
        self.assertEqual('sssv2', sot.service.service_type)

        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_list)
        self.assertFalse(sot.put_create)

    def test_make_token(self):
        sot = token.Token(**BASIC_EXAMPLE)
        self.assertEqual(BASIC_EXAMPLE['methods'], sot.methods)
        self.assertEqual(BASIC_EXAMPLE['user'], sot.user)
        self.assertEqual(BASIC_EXAMPLE['issued_at'], sot.issued_at)
        self.assertEqual(BASIC_EXAMPLE['expires_at'], sot.expires_at)
        self.assertEqual(BASIC_EXAMPLE['audit_ids'], sot.audit_ids)
        self.assertEqual(BASIC_EXAMPLE['project'], sot.project)
        self.assertEqual(BASIC_EXAMPLE['is_domain'], sot.is_domain)
        self.assertEqual(BASIC_EXAMPLE['roles'], sot.roles)
        self.assertEqual(BASIC_EXAMPLE['catalog'], sot.catalog)
        self.assertEqual(BASIC_EXAMPLE['X-Subject-Token'], sot.subject_token)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.network.v2 import mec

EXAMPLE = {
    "description": "Example gateway 1 description.",
    "id": "id12345678900",
    "name": "Example port 1",
    "qos_option_id": "qosid12345678",
    "tenant_id": "IDENTIFIER",
    "status": "ACTIVE",
    "mec_service_id": "mecsid12345678",
}


class TestPort(testtools.TestCase):

    def test_basic(self):
        sot = mec.MECGateway()
        self.assertEqual('mec_gateway', sot.resource_key)
        self.assertEqual('mec_gateways', sot.resources_key)
        self.assertEqual('/v2.0/mec_gateways', sot.base_path)
        self.assertEqual('network', sot.service.service_type)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = mec.MECGateway(**EXAMPLE)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['qos_option_id'], sot.qos_option_id)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['tenant_id'], sot.tenant_id)
        self.assertEqual(EXAMPLE['mec_service_id'], sot.mec_service_id)



from . import base

from ecl import resource2
from ecl.mvna import mvna_service


class SystemUpdate(resource2.Resource):
    resource_key = "system_update"
    resources_key = "system_updates"
    service = mvna_service.MVNAService("v1.0")
    base_path = '/' + service.version + '/system_updates'

    _query_mapping = base.MVNAQueryParameters(
        "id",
        "name",
        "description",
        "href",
        "current_revision",
        "next_revision",
        "applicable",
        "latest"
    )

    # Capabilities
    allow_list = True
    allow_get = True
    allow_create = False
    allow_update = False
    allow_delete = False
    patch_update = False

    # Properties
    #: It identifies connection resource uniquely
    id = resource2.Body('id')
    #: Name of system update
    name = resource2.Body('name')
    #: Description of system update
    description = resource2.Body('description')
    #: href of system update
    href = resource2.Body('href')
    #: Publish datetime of system update
    publish_datetime = resource2.Body('publish_datetime')
    #: Limit datetime of system update
    limit_datetime = resource2.Body('limit_datetime')
    #: Current revision of system update
    current_revision = resource2.Body('current_revision')
    #: Next revision of system update
    next_revision = resource2.Body('next_revision')
    #: Applicable of system update
    applicable = resource2.Body('applicable')



from ecl import resource2


class MVNABaseResource(resource2.Resource):

    def get_resource(self, session, resource_id, changes=None):
        if changes is not None and type(changes) is bool:
            uri = self.base_path + '/%s?changes=%s' % \
                  (resource_id, str(changes).lower())
        else:
            uri = self.base_path + '/%s' % resource_id
        resp = session.get(uri, endpoint_filter=self.service)
        self._translate_response(resp, has_body=True)
        return self

    def get_staged_configuration_resource(self, session, resource_id):
        uri = self.base_path + '/%s' % resource_id + '/staged'
        resp = session.get(uri, endpoint_filter=self.service)
        self._translate_response(resp, has_body=True)
        return self

    def create_staged_configuration(self, session, resource_id, **body):
        uri = self.base_path + '/%s/staged' % resource_id
        resp = session.post(uri, endpoint_filter=self.service,
                            json={self.resource_key: body})
        self._translate_response(resp, has_body=True)
        return self

    def update_staged_configuration(self, session, resource_id, **body):
        uri = self.base_path + '/%s/staged' % resource_id
        resp = session.patch(uri, endpoint_filter=self.service,
                             json={self.resource_key: body})
        self._translate_response(resp, has_body=True)
        return self

    def cancel_staged_configuration(self, session, resource_id):
        uri = self.base_path + '/%s/staged' % resource_id
        resp = session.delete(uri, endpoint_filter=self.service)
        self._translate_response(resp, has_body=False)
        return self


class MVNAQueryParameters(resource2.QueryParameters):
    def _transpose(self, query):
        result = {}
        for key, value in self._mapping.items():
            if key in query:
                normalized = str(query[key]).lower()
                if normalized == "true" or normalized == "false":
                    result[value] = normalized
                else:
                    result[value] = query[key]
        return result



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl import resource2
from ecl.sss import sss_service


class Token(resource2.Resource):
    resources_key = None
    resource_key = 'token'
    base_path = '/tokens/users'
    service = sss_service.SssAdminService()

    # Capabilities
    allow_create = True

    # Properties
    #: The authentication method.
    methods = resource2.Body('methods')
    #: A user object.
    user = resource2.Body('user')
    #: The date and time when the token was issued.
    issued_at = resource2.Body('issued_at')
    #: The date and time when the token expires.
    expires_at = resource2.Body('expires_at')
    #: A list of audit IDs.
    audit_ids = resource2.Body('audit_ids')
    #: A project object including the id, name and domain object representing
    #: the project the token is scoped to.
    project = resource2.Body('project')
    # Whether a project is acting as a domain.
    is_domain = resource2.Body('is_domain')
    #: A list of role objects.
    roles = resource2.Body('roles')
    #: A catalog object.
    catalog = resource2.Body('catalog')
    #: The authentication token. An authentication response returns
    #: the token ID in this header rather than in the response body.
    subject_token = resource2.Header('X-Subject-Token')

    def get_user_token_by_admin(self, session, user_id,
                                tenant_id=None, no_catalog=True):
        """Obtain a user token by SSS on behalf

        :param session: The session to use for making this request.
        :type session: :class:`~ecl.session.Session`
        :param user_id: ID of a user
        :param tenant_id: ID of a tenant
        :param bool no_catalog: Exclude the service catalog from the response.
        :return: :class:`~ecl.sss.v2.token.Token`
        """
        url = '%s/%s' % (self.base_path, user_id)
        if no_catalog:
            url += '?nocatalog=true'
        body = {"tenant_id": tenant_id} if tenant_id else {}

        resp = session.post(url, endpoint_filter=self.service, json=body)
        self._translate_response(resp, has_body=True)
        return self



from . import base

from ecl import resource2
from ecl.mvna import mvna_service


class Policy(base.MVNABaseResource):
    resource_key = "policy"
    resources_key = "policies"
    service = mvna_service.MVNAService("v1.0")
    base_path = '/' + service.version + '/policies'

    _query_mapping = base.MVNAQueryParameters(
        "id",
        "name",
        "description",
        "configuration_status",
        "operation_status",
        "algorithm",
        "persistence",
        "idle_timeout",
        "sorry_page_url",
        "source_nat",
        "certificate_id",
        "health_monitor_id",
        "listener_id",
        "default_target_group_id",
        "tls_policy_id",
        "load_balancer_id",
        "tenant_id"
    )

    # Capabilities
    allow_list = True
    allow_get = True
    allow_create = True
    allow_update = True
    allow_delete = True
    patch_update = True

    # Properties
    #: It identifies connection resource uniquely
    id = resource2.Body('id')
    #: Name of policy
    name = resource2.Body('name')
    #: Description of policy
    description = resource2.Body('description')
    #: Tags of policy
    tags = resource2.Body('tags')
    #: Configuration status of policy
    configuration_status = resource2.Body('configuration_status')
    #: Operation status of policy
    operation_status = resource2.Body('operation_status')
    #: Algorithm of policy
    algorithm = resource2.Body('algorithm')
    #: persistence of policy
    persistence = resource2.Body('persistence')
    #: Idle Timeout of policy
    idle_timeout = resource2.Body('idle_timeout')

    #: Sorry page URL of policy
    sorry_page_url = resource2.Body('sorry_page_url')
    #: Source NAT of policy
    source_nat = resource2.Body('source_nat')
    #: Certificate ID of policy
    certificate_id = resource2.Body('certificate_id')
    #: TLS policy ID of policy
    tls_policy_id = resource2.Body('tls_policy_id')

    #: Health monitor ID of policy
    health_monitor_id = resource2.Body('health_monitor_id')
    #: Listener ID of policy
    listener_id = resource2.Body('listener_id')
    #: Default target group ID of policy
    default_target_group_id = resource2.Body('default_target_group_id')
    #: Load balancer ID of policy
    load_balancer_id = resource2.Body('load_balancer_id')
    #: Tenant ID of policy
    tenant_id = resource2.Body('tenant_id')
    #: Current configuration of policy
    current = resource2.Body('current')
    #: Staged configuration of policy
    staged = resource2.Body('staged')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.baremetal.v2 import chassis

IDENTIFIER = 'IDENTIFIER'
CHASSIS_EXAMPLE = {
    'id': IDENTIFIER,
    'availability_zone': '2',
    'flavor_name': '3',
    'hardware_summary': {'4': 4},
    'status': '5',
    'server_id': '6',
    'server_name': '7',
    'contract_year': 8,
    'start_time': '9',
}

CHASSIS_DETAIL_EXAMPLE = {
    'id': IDENTIFIER,
    'availability_zone': '2',
    'flavor_name': '3',
    'hardware_summary': {'4': 4},
    'status': '5',
    'server_id': '6',
    'server_name': '7',
    'contract_year': 8,
    'start_time': '9',
    'cpus': [{'10': 10},],
    'disks': [{'11': 11},],
    'rams': [{'12': 12},],
}
from unittest.mock import patch

class TestChassis(testtools.TestCase):

    def test_basic(self):
        sot = chassis.Chassis()
        self.assertEqual('chassis', sot.resource_key)
        self.assertEqual('chassis', sot.resources_key)
        self.assertEqual('/chassis', sot.base_path)
        self.assertEqual('baremetal-server', sot.service.service_type)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_head)

    def test_make_basic(self):
        sot = chassis.Chassis(**CHASSIS_EXAMPLE)
        self.assertEqual(CHASSIS_EXAMPLE['id'], sot.id)
        self.assertEqual(CHASSIS_EXAMPLE['availability_zone'], sot.availability_zone)
        self.assertEqual(CHASSIS_EXAMPLE['flavor_name'], sot.flavor_name)
        self.assertEqual(CHASSIS_EXAMPLE['hardware_summary'], sot.hardware_summary)
        self.assertEqual(CHASSIS_EXAMPLE['status'], sot.status)
        self.assertEqual(CHASSIS_EXAMPLE['server_id'], sot.server_id)
        self.assertEqual(CHASSIS_EXAMPLE['server_name'], sot.server_name)
        self.assertEqual(CHASSIS_EXAMPLE['contract_year'], sot.contract_year)
        self.assertEqual(CHASSIS_EXAMPLE['start_time'], sot.start_time)

    def test_detail(self):
        sot = chassis.ChassisDetail()
        self.assertEqual('chassis', sot.resource_key)
        self.assertEqual('chassis', sot.resources_key)
        self.assertEqual('/chassis/detail', sot.base_path)
        self.assertEqual('baremetal-server', sot.service.service_type)
        self.assertFalse(sot.allow_get)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_head)

    def test_make_detail(self):
        sot = chassis.ChassisDetail(**CHASSIS_DETAIL_EXAMPLE)
        self.assertEqual(CHASSIS_DETAIL_EXAMPLE['id'], sot.id)
        self.assertEqual(CHASSIS_DETAIL_EXAMPLE['availability_zone'], sot.availability_zone)
        self.assertEqual(CHASSIS_DETAIL_EXAMPLE['flavor_name'], sot.flavor_name)
        self.assertEqual(CHASSIS_DETAIL_EXAMPLE['hardware_summary'], sot.hardware_summary)
        self.assertEqual(CHASSIS_DETAIL_EXAMPLE['status'], sot.status)
        self.assertEqual(CHASSIS_DETAIL_EXAMPLE['server_id'], sot.server_id)
        self.assertEqual(CHASSIS_DETAIL_EXAMPLE['server_name'], sot.server_name)
        self.assertEqual(CHASSIS_DETAIL_EXAMPLE['contract_year'], sot.contract_year)
        self.assertEqual(CHASSIS_DETAIL_EXAMPLE['start_time'], sot.start_time)
        self.assertEqual(CHASSIS_DETAIL_EXAMPLE['cpus'], sot.cpus)
        self.assertEqual(CHASSIS_DETAIL_EXAMPLE['disks'], sot.disks)
        self.assertEqual(CHASSIS_DETAIL_EXAMPLE['rams'], sot.rams)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
import unittest

import six

import ecl
from ecl.tests.functional import base


class TestToken(base.BaseFunctionalTest):
    """Test token resources

    To run this test, you will need.
    - Prior Smart Data Platform login
    - Administrator account
    """
    @classmethod
    def setUpClass(cls):
        # keystone admin endpoint URL
        admin_keystone_url = os.getenv('ADMIN_KEYSTONE_URL')
        # adminstrator account
        admin_username = os.getenv('ADMIN_USER_NAME')
        admin_password = os.getenv('ADMIN_PASSWORD')
        # user information
        cls.user_ecid = os.getenv('USER_ECID')
        cls.user_tenant_id = os.getenv('UESR_TENANT_ID')

        cls.conn = ecl.connection.Connection(auth_url=admin_keystone_url,
                                             username=admin_username,
                                             password=admin_password,
                                             user_domain_id='default',
                                             timeout=100)

    @unittest.skip("Skip as it requires administrator authorization.")
    def test_get_user_token_by_admin(self):
        token = self.conn.sss.get_user_token_by_admin(self.user_ecid)

        self.assertIsInstance(token.methods, list)
        self.assertIsInstance(token.user, dict)
        self.assertIsInstance(token.issued_at, six.string_types)
        self.assertIsInstance(token.expires_at, six.string_types)
        self.assertIsInstance(token.audit_ids, list)
        self.assertIsNone(token.project)
        self.assertIsNone(token.is_domain)
        self.assertIsNone(token.roles)
        self.assertIsNone(token.catalog)
        self.assertIsInstance(token.subject_token, six.string_types)

    @unittest.skip("Skip as it requires administrator authorization.")
    def test_get_user_token_by_admin_with_catalog(self):
        token = self.conn.sss.get_user_token_by_admin(
            self.user_ecid, tenant_id=self.user_tenant_id, no_catalog=False)

        self.assertIsInstance(token.methods, list)
        self.assertIsInstance(token.user, dict)
        self.assertIsInstance(token.issued_at, six.string_types)
        self.assertIsInstance(token.expires_at, six.string_types)
        self.assertIsInstance(token.audit_ids, list)
        self.assertIsInstance(token.project, dict)
        self.assertIsInstance(token.is_domain, bool)
        self.assertIsInstance(token.roles, list)
        self.assertIsInstance(token.catalog, list)
        self.assertIsInstance(token.subject_token, six.string_types)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.sss import sss_service
from ecl import resource2
from ecl import utils

class Workspace(resource2.Resource):
    resource_key = None
    resources_key = 'workspaces'
    base_path = '/workspaces'
    service = sss_service.SssService()

    # Capabilities
    allow_create = True
    allow_get = True
    allow_list = True
    allow_delete = True
    allow_update = True

    _query_mapping = resource2.QueryParameters("contract_id")

    # Properties
    #: ID of the contract.
    contract_id = resource2.Body('contract_id')
    #: ID of the workspace.
    workspace_id = resource2.Body('workspace_id', alternate_id=True)
    #: Name of the workspace. Workspace name is also a unique identifier of workspaces.
    workspace_name = resource2.Body('workspace_name')
    #: Description of the workspace.
    description = resource2.Body('description')
    #: Workspace created time.
    start_time = resource2.Body('start_time')
    #: list of workspace.
    workspaces = resource2.Body('workspaces')
    #: Array of region information where tenants can be created.
    regions = resource2.Body('regions')
    #: ID of the user.
    user_id = resource2.Body('user_id')
    #: list of user.
    users = resource2.Body('users')

    def update(self, session, workspace_id, **attrs):
        """Updates the description for the designated workspace."""

        uri = utils.urljoin(self.base_path, workspace_id)
        resp = session.put(
            uri, endpoint_filter=self.service, json=attrs)
        self._translate_response(resp, has_body=False)
        return self

    def add_workspace_role_assignment(self, session, user_id, workspace_id):
        """Add role between user and workspace."""

        url = '/workspace-roles'
        resp = session.post(url, endpoint_filter=self.service, json={"user_id": user_id, "workspace_id": workspace_id})
        self._translate_response(resp, has_body=True)
        return self

    def delete_workspace_role_assignment(self, session, workspace_id, user_id):
        """Delete role between user and workspace."""

        url = '/workspace-roles/workspaces/%s/users/%s' % (workspace_id, user_id)
        resp = session.delete(url, endpoint_filter=self.service)
        self._translate_response(resp, has_body=False)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.network.v2 import mec

EXAMPLE = {
    "description": "Example service 1 description.",
    "id": "id12345678900",
    "name": "Example port 1",
    "tenant_id": "IDENTIFIER",
    "zone": "Name",
}


class TestPort(testtools.TestCase):

    def test_basic(self):
        sot = mec.MECService()
        self.assertEqual('mec_service', sot.resource_key)
        self.assertEqual('mec_services', sot.resources_key)
        self.assertEqual('/v2.0/mec_services', sot.base_path)
        self.assertEqual('network', sot.service.service_type)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = mec.MECService(**EXAMPLE)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['zone'], sot.zone)
        self.assertEqual(EXAMPLE['tenant_id'], sot.tenant_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six

from ecl.tests.functional import base


class TestChassis(base.BaseFunctionalTest):

    def test_chassis(self):
        chassis_list = list(self.conn.baremetal.chassis(details=False))
        self.assertGreater(len(chassis_list), 0)

        for chassis in chassis_list:
            print(chassis)
            self.assertIsInstance(chassis.id, six.string_types)
            self.assertIsInstance(chassis.availability_zone, six.string_types)
            self.assertIsInstance(chassis.flavor_name, six.string_types)
            self.assertIsInstance(chassis.hardware_summary, dict)
            self.assertIsInstance(chassis.status, six.string_types)
            self.assertIsInstance(chassis.server_id,
                                  (six.string_types, type(None)))
            self.assertIsInstance(chassis.server_name,
                                  (six.string_types, type(None)))
            self.assertIsInstance(chassis.contract_year, int)
            self.assertIsInstance(chassis.start_time, six.string_types)

    def test_chassis_detail(self):
        chassis_list = list(self.conn.baremetal.chassis(details=True))
        self.assertGreater(len(chassis_list), 0)

        for chassis in chassis_list:
            print(chassis)
            self.assertIsInstance(chassis.id, six.string_types)
            self.assertIsInstance(chassis.availability_zone, six.string_types)
            self.assertIsInstance(chassis.flavor_name, six.string_types)
            self.assertIsInstance(chassis.hardware_summary, dict)
            self.assertIsInstance(chassis.status, six.string_types)
            self.assertIsInstance(chassis.server_id,
                                  (six.string_types, type(None)))
            self.assertIsInstance(chassis.server_name,
                                  (six.string_types, type(None)))
            self.assertIsInstance(chassis.contract_year, int)
            self.assertIsInstance(chassis.start_time, six.string_types)

            self.assertIsInstance(chassis.cpus, list)
            self.assertIsInstance(chassis.disks, list)
            self.assertIsInstance(chassis.rams, list)

    def test_get_chassis(self):
        # ChassisIDはMock Serverに合わせて変更する
        chassis_id = '31e7a0c4-f49e-19e6-2192-c9f9cc6f5a74'

        chassis = self.conn.baremetal.get_chassis(chassis_id)
        print(chassis)
        # Mock Serverを使用するとランダムなUUIDが返却されるのでパスパラメータを一致しない
        # self.assertEqual(chassis.id, chassis_id)

        self.assertIsInstance(chassis.availability_zone, six.string_types)
        self.assertIsInstance(chassis.flavor_name, six.string_types)
        self.assertIsInstance(chassis.hardware_summary, dict)
        self.assertIsInstance(chassis.status, six.string_types)
        self.assertIsInstance(chassis.server_id,
                              (six.string_types, type(None)))
        self.assertIsInstance(chassis.server_name,
                              (six.string_types, type(None)))
        self.assertIsInstance(chassis.contract_year, int)
        self.assertIsInstance(chassis.start_time, six.string_types)

        self.assertIsInstance(chassis.cpus, list)
        self.assertIsInstance(chassis.disks, list)
        self.assertIsInstance(chassis.rams, list)
    


# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
import unittest

import six

import ecl
from ecl.tests.functional import base


class TestServiceMenu(base.BaseFunctionalTest):
    """Test ServiceMenu resources

    To run this test, you will need.
    - Prior Smart Data Platform login
    - Administrator account
    """
    @classmethod
    def setUpClass(cls):
        # keystone admin endpoint URL
        admin_keystone_url = os.getenv('ADMIN_KEYSTONE_URL')
        # adminstrator account
        admin_username = os.getenv('ADMIN_USER_NAME')
        admin_password = os.getenv('ADMIN_PASSWORD')
        # user information
        cls.user_ecid = os.getenv('USER_ECID')

        cls.conn = ecl.connection.Connection(auth_url=admin_keystone_url,
                                             username=admin_username,
                                             password=admin_password,
                                             user_domain_id='default',
                                             timeout=100)

    @unittest.skip("Skip as it requires administrator authorization.")
    def test_get_user_service_menu(self):
        service_menu = self.conn.sss.get_user_service_menu(self.user_ecid)
        self.assertIsInstance(service_menu.user_id, six.string_types)
        self.assertIsInstance(service_menu.categories, list)

    @unittest.skip("Skip as it requires administrator authorization.")
    def test_categories_in_service_menu(self):
        service_menu = self.conn.sss.get_user_service_menu(self.user_ecid)
        for category in service_menu.categories:
            self._check_category(category)
            sub_categories = category.get('sub_categories')

            for sub_category in sub_categories:
                self._check_sub_category(sub_category)
                service_menus = sub_category.get('service_menus')

                for service_menu in service_menus:
                    self._check_service_menu(service_menu)

    def _check_category(self, category):
        self.assertIsInstance(category, dict)
        self.assertIsInstance(category.get('category_name'),
                              six.string_types)
        self.assertIsInstance(category.get('sub_categories'), list)

    def _check_sub_category(self, sub_category):
        self.assertIsInstance(sub_category, dict)
        self.assertIsInstance(sub_category.get('sub_category_name'),
                              six.string_types)
        self.assertIsInstance(sub_category.get('service_menus'), list)

    def _check_service_menu(self, service_menu):
        self.assertIsInstance(service_menu, dict)
        self.assertIsInstance(service_menu.get('menu_name'), six.string_types)
        areas = service_menu.get('areas')
        self.assertIsInstance(areas, list)
        for area in areas:
            self.assertIsInstance(area, dict)
            self.assertIn('area_name', area)
            self.assertIn('region_name', area)



from . import base

from ecl import exceptions
from ecl import resource2
from ecl.mvna import mvna_service


class LoadBalancer(base.MVNABaseResource):
    resource_key = "load_balancer"
    resources_key = "load_balancers"
    service = mvna_service.MVNAService("v1.0")
    base_path = '/' + service.version + '/load_balancers'

    _query_mapping = base.MVNAQueryParameters(
        "id",
        "name",
        "description",
        "configuration_status",
        "monitoring_status",
        "operation_status",
        "primary_availability_zone",
        "secondary_availability_zone",
        "active_availability_zone",
        "revision",
        "plan_id",
        "tenant_id"
    )

    # Capabilities
    allow_list = True
    allow_get = True
    allow_create = True
    allow_update = True
    allow_delete = True
    patch_update = True

    # Properties
    #: It identifies connection resource uniquely
    id = resource2.Body('id')
    #: Name of load balancer
    name = resource2.Body('name')
    #: Description of load balancer
    description = resource2.Body('description')
    #: Tags of load balancer
    tags = resource2.Body('tags')
    #: Configuration status of load balancer
    configuration_status = resource2.Body('configuration_status')
    #: Monitoring status of load balancer
    monitoring_status = resource2.Body('monitoring_status')
    #: Operation status of load balancer
    operation_status = resource2.Body('operation_status')
    #: Primary availability zone of load balancer
    primary_availability_zone = resource2.Body('primary_availability_zone')
    #: Secondary availability zone of load balancer
    secondary_availability_zone = resource2.Body('secondary_availability_zone')
    #: Active availability zone of load balancer
    active_availability_zone = resource2.Body('active_availability_zone')
    #: Revision of load balancer
    revision = resource2.Body('revision')
    #: Plan ID of load balancer
    plan_id = resource2.Body('plan_id')
    #: Plan name of load balancer
    plan_name = resource2.Body('plan_name')
    #: Tenant ID of load balancer
    tenant_id = resource2.Body('tenant_id')
    #: Syslog servers of load balancer
    syslog_servers = resource2.Body('syslog_servers')
    #: Interfaces of load balancer
    interfaces = resource2.Body('interfaces')
    #: Current configuration of load balancer
    current = resource2.Body('current')
    #: Staged configuration of load balancer
    staged = resource2.Body('staged')

    def delete_resource(self, session, resource_id, x_mvna_request_id=None,
                        ignore_missing=False):
        uri = self.base_path + '/%s' % resource_id

        try:
            if x_mvna_request_id:
                resp = session.delete(
                    uri, endpoint_filter=self.service,
                    headers={"X-MVNA-Request-Id": x_mvna_request_id})
            else:
                resp = session.delete(uri, endpoint_filter=self.service)
        except exceptions.NotFoundException:
            if ignore_missing:
                return None
            raise exceptions.ResourceNotFound(
                message="No %s found for %s" %
                        (self.__class__.__name__, resource_id))

        self._translate_response(resp, has_body=False)
        return self

    def action(self, session, resource_id, x_mvna_request_id=None, **body):
        uri = self.base_path + '/%s/action' % resource_id
        if x_mvna_request_id:
            resp = session.post(
                uri, endpoint_filter=self.service,
                headers={"X-MVNA-Request-Id": x_mvna_request_id}, json=body)
        else:
            resp = session.post(uri, endpoint_filter=self.service, json=body)
        self._translate_response(resp, has_body=False)
        return self



from . import base

from ecl import resource2
from ecl.mvna import mvna_service


class Plan(resource2.Resource):
    resource_key = "plan"
    resources_key = "plans"
    service = mvna_service.MVNAService("v1.0")
    base_path = '/' + service.version + '/plans'

    _query_mapping = base.MVNAQueryParameters(
        "id",
        "name",
        "description",
        "bandwidth",
        "redundancy",
        "max_number_of_interfaces",
        "max_number_of_health_monitors",
        "max_number_of_listeners",
        "max_number_of_policies",
        "max_number_of_routes",
        "max_number_of_target_groups",
        "max_number_of_rules",
        "max_number_of_conditions",
        "max_number_of_members",
        "enabled"
    )

    # Capabilities
    allow_list = True
    allow_get = True
    allow_create = False
    allow_update = False
    allow_delete = False
    patch_update = False

    # Properties
    #: It identifies connection resource uniquely
    id = resource2.Body('id')
    #: Name of plan
    name = resource2.Body('name')
    #: Description of plan
    description = resource2.Body('description')
    #: Bandwidth of plan
    bandwidth = resource2.Body('bandwidth')
    #: Redundancy of plan
    redundancy = resource2.Body('redundancy')
    #: Max number of interfaces
    max_number_of_interfaces = resource2.Body('max_number_of_interfaces')
    #: Max number of health monitors
    max_number_of_health_monitors = \
        resource2.Body('max_number_of_health_monitors')
    #: Max number of listeners
    max_number_of_listeners = resource2.Body('max_number_of_listeners')
    #: Max number of policies
    max_number_of_policies = resource2.Body('max_number_of_policies')
    #: Max numbers of routes
    max_number_of_routes = resource2.Body('max_number_of_routes')
    #: Max number of target groups
    max_number_of_target_groups = resource2.Body('max_number_of_target_groups')
    #: Max number of rules
    max_number_of_rules = resource2.Body('max_number_of_rules')
    #: Max number of conditions
    max_number_of_conditions = resource2.Body('max_number_of_conditions')
    #: Max number of members
    max_number_of_members = resource2.Body('max_number_of_members')
    #: Enabled or disabled
    enabled = resource2.Body('enabled')



from . import base

from ecl import resource2
from ecl.mvna import mvna_service


class TLSPolicy(resource2.Resource):
    resource_key = "tls_policy"
    resources_key = "tls_policies"
    service = mvna_service.MVNAService("v1.0")
    base_path = '/' + service.version + '/tls_policies'

    _query_mapping = base.MVNAQueryParameters(
        "id", "name", "description", "default",
    )

    # Capabilities
    allow_list = True
    allow_get = True
    allow_create = False
    allow_update = False
    allow_delete = False
    patch_update = False

    # Properties
    #: It identifies connection resource uniquely
    id = resource2.Body('id')
    #: Name of tls policy
    name = resource2.Body('name')
    #: Description of tls policy
    description = resource2.Body('description')
    #: Default of tls policy
    default = resource2.Body('default')
    #: TLS protocols of tls policy
    tls_protocols = resource2.Body('tls_protocols')
    #: TLS Ciphers of tls policy
    tls_ciphers = resource2.Body('tls_ciphers')



from ecl import proxy2
from ecl.mvna.v1 import certificate as _certificate
from ecl.mvna.v1 import health_monitor as _health_monitor
from ecl.mvna.v1 import listener as _listener
from ecl.mvna.v1 import load_balancer as _load_balancer
from ecl.mvna.v1 import system_update as _system_update
from ecl.mvna.v1 import operation as _operation
from ecl.mvna.v1 import plan as _plan
from ecl.mvna.v1 import policy as _policy
from ecl.mvna.v1 import route as _route
from ecl.mvna.v1 import rule as _rule
from ecl.mvna.v1 import target_group as _target_group
from ecl.mvna.v1 import tls_policy as _tls_policy


class Proxy(proxy2.BaseProxy):

    def load_balancers(self, **params):
        """List Managed Load Balancers."""
        return list(self._list(_load_balancer.LoadBalancer, paginated=False,
                               **params))

    def create_load_balancer(self, plan_id, interfaces,
                             name=None, description=None, tags=None,
                             syslog_servers=None):
        """Create Managed Load Balancer.

        :param string plan_id: Plan ID of Managed Load Balancer
        :param list interfaces: Interface of Managed Load Balancer
        :param string name: Name of Managed Load Balancer
        :param string description: Description of Managed Load Balancer
        :param dict tags: Tags of Managed Load Balancer
        :param list syslog_servers: Syslog Servers of Managed Load Balancer
        :return: Managed Load Balancer
        """
        body = {"plan_id": plan_id, "interfaces": interfaces}
        if name is not None:
            body["name"] = name
        if description is not None:
            body["description"] = description
        if tags is not None:
            body["tags"] = tags
        if syslog_servers is not None:
            body["syslog_servers"] = syslog_servers
        return self._create(_load_balancer.LoadBalancer, **body)

    def get_load_balancer(self, load_balancer_id, changes=None):
        """Retrieve Managed Load Balancer Information.

        :param string load_balancer_id: ID of Managed Load Balancer
        :param bool changes: Whether or not to retrieve staged configuration
        :return: Managed Load Balancer
        """
        load_balancer = _load_balancer.LoadBalancer()
        return load_balancer.get_resource(self.session, load_balancer_id,
                                          changes)

    def update_load_balancer(self, load_balancer_id, **params):
        """Update Managed Load Balancer Attributes.

        :param string load_balancer_id: ID of Managed Load Balancer
        :attrs **params: Parameters for load balancer update.

            * string name: Name of Managed Load Balancer
            * string description: Description of Managed Load Balancer
            * dict tags: Tags of Managed Load Balancer

        :return: Managed Load Balancer
        """
        return self._update(_load_balancer.LoadBalancer, load_balancer_id,
                            **params)

    def delete_load_balancer(self, load_balancer_id, x_mvna_request_id=None,
                             ignore_missing=False):
        """Delete Managed Load Balancer.

        :param string load_balancer_id: ID of Managed Load Balancer
        :param string x_mvna_request_id: ID to identify the transaction history
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent load balancer.
        :return: None
        """
        load_balancer = _load_balancer.LoadBalancer()
        load_balancer.delete_resource(self.session, load_balancer_id,
                                      x_mvna_request_id, ignore_missing)

    def action_load_balancer(self, load_balancer_id, x_mvna_request_id=None,
                             **body):
        """Reflect the configuration to Managed Load Balancer.

        :param string load_balancer_id: ID of Managed Load Balancer
        :param string x_mvna_request_id: ID to identify the transaction history
        :param dict body: Request Body which want to apply.
        :return: None
        """
        load_balancer = _load_balancer.LoadBalancer()
        load_balancer.action(self.session, load_balancer_id, x_mvna_request_id,
                             **body)

    def create_staged_load_balancer_configuration(self,
                                                  load_balancer_id,
                                                  syslog_servers=None,
                                                  interfaces=None):
        """Create Staged Managed Load Balancer Configuration.

        :param string load_balancer_id: ID of Managed Load Balancer
        :param list syslog_servers: Syslog Servers of Managed Load Balancer
        :param list interfaces: Interface of Managed Load Balancer
        :return: Managed Load Balancer
        """
        body = {}
        if syslog_servers is not None:
            body["syslog_servers"] = syslog_servers
        if interfaces is not None:
            body["interfaces"] = interfaces

        load_balancer = _load_balancer.LoadBalancer()
        return load_balancer.create_staged_configuration(self.session,
                                                         load_balancer_id,
                                                         **body)

    def get_staged_load_balancer_configuration(self, load_balancer_id):
        """Retrieve Staged Managed Load Balancer Configuration.

        :param string load_balancer_id: ID of Managed Load Balancer
        :return: Managed Load Balancer
        """
        load_balancer = _load_balancer.LoadBalancer()
        return load_balancer.get_staged_configuration_resource(self.session, load_balancer_id)

    def update_staged_load_balancer_configuration(self,
                                                  load_balancer_id,
                                                  syslog_servers=None,
                                                  interfaces=None):
        """Update Staged Managed Load Balancer Configuration.

        :param string load_balancer_id: ID of Managed Load Balancer
        :param list syslog_servers: Syslog Servers of Managed Load Balancer
        :param list interfaces: Interface of Managed Load Balancer
        :return: Managed Load Balancer
        """
        body = {}
        if syslog_servers is not None:
            body["syslog_servers"] = syslog_servers
        if interfaces is not None:
            body["interfaces"] = interfaces

        load_balancer = _load_balancer.LoadBalancer()
        return load_balancer.update_staged_configuration(self.session,
                                                         load_balancer_id,
                                                         **body)

    def cancel_staged_load_balancer_configuration(self, load_balancer_id):
        """Delete Staged Managed Load Balancer Configuration.

        :param string load_balancer_id: ID of Managed Load Balancer
        :return: None
        """
        load_balancer = _load_balancer.LoadBalancer()
        load_balancer.cancel_staged_configuration(self.session,
                                                  load_balancer_id)

    def target_groups(self, **params):
        """List Target Groups."""
        return list(self._list(
            _target_group.TargetGroup, paginated=False, **params))

    def create_target_group(self, load_balancer_id, members,
                            name=None, description=None, tags=None):
        """Create Target Group.

        :param string load_balancer_id: Load Balancer ID of Target Group
        :param string members: Members of Target Group
        :param string name: Name of Target Group
        :param string description: Description of Target Group
        :param dict tags: Tags of Target Group
        :return: Target Group
        """
        body = {
            'load_balancer_id': load_balancer_id,
            'members': members
        }
        if name is not None:
            body["name"] = name
        if description is not None:
            body["description"] = description
        if tags is not None:
            body["tags"] = tags
        return self._create(_target_group.TargetGroup, **body)

    def get_target_group(self, target_group_id, changes=None):
        """Retrieve Target Group Information.

        :param string target_group_id: ID of Target Group
        :param bool changes: Whether or not to retrieve staged configuration
        :return: Target Group
        """
        target_group = _target_group.TargetGroup()
        return target_group.get_resource(self.session, target_group_id,
                                         changes)

    def update_target_group(self, target_group_id, **params):
        """Update Target Group Attributes.

        :param string load_balancer_id: ID of Managed Load Balancer
        :attrs **params: Parameters for target group update.

            * string name: Name of Managed Load Balancer
            * string description: Description of Managed Load Balancer
            * dict tags: Tags of Managed Load Balancer

        :return: Target Group
        """
        return self._update(_target_group.TargetGroup, target_group_id,
                            **params)

    def delete_target_group(self, target_group_id, ignore_missing=False):
        """Delete Target Group.

        :param string target_group_id: ID of Target Group
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent target group.
        :return: None
        """
        self._delete(_target_group.TargetGroup, target_group_id,
                     ignore_missing=ignore_missing)

    def create_staged_target_group_configuration(self, target_group_id,
                                                 members=None):
        """Create Staged Target Group Configuration.

        :param string target_group_id: ID of Target Group
        :param string members: Members of Target Group
        :return: Target Group
        """
        body = {}
        if members is not None:
            body["members"] = members

        target_group = _target_group.TargetGroup()
        return target_group.create_staged_configuration(self.session,
                                                        target_group_id,
                                                        **body)

    def get_staged_target_group_configuration(self, target_group_id):
        """Retrieve Staged Target Group Configuration.

        :param string target_group_id: ID of Target Group
        :return: Target Group
        """
        target_group = _target_group.TargetGroup()
        return target_group.get_staged_configuration_resource(self.session, target_group_id)

    def update_staged_target_group_configuration(self, target_group_id,
                                                 members=None):
        """Update Staged Target Group Configuration.

        :param string target_group_id: ID of Target Group
        :param string members: Members of Target Group
        :return: Target Group
        """
        body = {}
        if members is not None:
            body["members"] = members

        target_group = _target_group.TargetGroup()
        return target_group.update_staged_configuration(self.session,
                                                        target_group_id,
                                                        **body)

    def cancel_staged_target_group_configuration(self, target_group_id):
        """Delete Staged Target Group Configuration.

        :param string target_group_id: ID of Target Group
        :return: None
        """
        target_group = _target_group.TargetGroup()
        target_group.cancel_staged_configuration(self.session, target_group_id)

    def certificates(self, **params):
        """List Certificates."""
        return list(self._list(
            _certificate.Certificate, paginated=False, **params))

    def create_certificate(self, name=None, description=None, tags=None):
        """Create Certificate.

        :param string name: Name of Certificate
        :param string description: Description of Certificate
        :param string tags: Tags of Certificate
        :return: Certificate
        """
        body = {}
        if name is not None:
            body["name"] = name
        if description is not None:
            body["description"] = description
        if tags is not None:
            body["tags"] = tags
        return self._create(_certificate.Certificate, **body)

    def get_certificate(self, certificate_id):
        """Retrieve Certificate Information.

        :param string certificate_id: ID of Certificate
        :return: Certificate
        """
        return self._get(_certificate.Certificate, certificate_id)

    def update_certificate(self, certificate_id,
                           name=None, description=None, tags=None):
        """Update Certificate Attributes.

        :param string certificate_id: ID of Certificate
        :param string name: Name of Certificate
        :param string description: Description of Certificate
        :param dict tags: Tags of Certificate
        :return: Certificate
        """
        body = {}
        if name is not None:
            body["name"] = name
        if description is not None:
            body["description"] = description
        if tags is not None:
            body["tags"] = tags
        return self._update(_certificate.Certificate, certificate_id,
                            **body)

    def delete_certificate(self, certificate_id, ignore_missing=False):
        """Delete Certificate.

        :param string certificate_id: ID of Certificate
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent certificate.
        :return: None
        """
        self._delete(_certificate.Certificate, certificate_id,
                     ignore_missing=ignore_missing)

    def upload_certificate(self,
                           certificate_id, certificate_type, certificate_content):
        """Upload the Certificate.

        :param string certificate_id: ID of Certificate
        :param string certificate_type: Type of Certificate
        :param string certificate_content: File content of Certificate
        :return: None
        """
        body = {'type': certificate_type, 'content': certificate_content}
        certificate = _certificate.Certificate()
        certificate.upload(self.session, certificate_id, **body)

    def listeners(self, **params):
        """List Listeners."""
        return list(self._list(
            _listener.Listener, paginated=False, **params))

    def create_listener(self, ip_address, port, protocol, load_balancer_id,
                        name=None, description=None, tags=None):
        """Create Listener.

        :param string ip_address: IP Address of Listener
        :param string port: Port of Listener
        :param string protocol: Protocol of Listener
        :param string load_balancer_id: Load Balancer ID of Listener
        :param string name: Name of Listener
        :param string description: Description of Listener
        :param dict tags: Tags of Listener

        :return: Listener
        """
        body = {
            'ip_address': ip_address,
            'port': port,
            'protocol': protocol,
            'load_balancer_id': load_balancer_id
        }
        if name is not None:
            body["name"] = name
        if description is not None:
            body["description"] = description
        if tags is not None:
            body["tags"] = tags
        return self._create(_listener.Listener, **body)

    def get_listener(self, listener_id, changes=None):
        """Retrieve Listener Information.

        :param string listener_id: ID of Listener
        :param bool changes: Whether or not to retrieve staged configuration
        :return: Listener
        """
        listener = _listener.Listener()
        return listener.get_resource(self.session, listener_id, changes)

    def update_listener(self, listener_id, **params):
        """Update Listener Attributes.

        :param string listener_id: ID of Listener

        :attrs **params: Parameters for listener update.

            * string name: Name of Listener
            * string description: Description of Listener
            * dict tags: Tags of Listener

        :return: Listener
        """
        return self._update(_listener.Listener, listener_id, **params)

    def delete_listener(self, listener_id, ignore_missing=False):
        """Delete Listener.

        :param string listener_id: ID of Listener
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent listener.
        :return: None
        """
        self._delete(_listener.Listener, listener_id,
                     ignore_missing=ignore_missing)

    def create_staged_listener_configuration(
            self, listener_id, ip_address=None, port=None, protocol=None):
        """Create Staged Listener Configuration.

        :param string listener_id: ID of Listener
        :param string ip_address: IP Address of Listener
        :param string port: Port of Listener
        :param string protocol: Protocol of Listener
        :return: Listener
        """
        body = {}
        if ip_address is not None:
            body["ip_address"] = ip_address
        if port is not None:
            body["port"] = port
        if protocol is not None:
            body["protocol"] = protocol

        listener = _listener.Listener()
        return listener.create_staged_configuration(self.session,
                                                    listener_id,
                                                    **body)

    def get_staged_listener_configuration(self, listener_id):
        """Retrieve Staged Listener Configuration.

        :param string listener_id: ID of Listener
        :return: Listener
        """
        listener = _listener.Listener()
        return listener.get_staged_configuration_resource(self.session, listener_id)

    def update_staged_listener_configuration(
            self, listener_id, ip_address=None, port=None, protocol=None):
        """Update Staged Listener Configuration.

        :param string listener_id: ID of Listener
        :param string ip_address: IP Address of Listener
        :param string port: Port of Listener
        :param string protocol: Protocol of Listener
        :return: Listener
        """
        body = {}
        if ip_address is not None:
            body["ip_address"] = ip_address
        if port is not None:
            body["port"] = port
        if protocol is not None:
            body["protocol"] = protocol

        listener = _listener.Listener()
        return listener.update_staged_configuration(self.session,
                                                    listener_id, **body)

    def cancel_staged_listener_configuration(self, listener_id):
        """Delete Staged Listener Configuration.

        :param string listener_id: ID of Listener
        :return: None
        """
        listener = _listener.Listener()
        listener.cancel_staged_configuration(self.session, listener_id)

    def system_updates(self, **params):
        """List System Updates."""
        return list(self._list(
            _system_update.SystemUpdate, paginated=False, **params))

    def get_system_update(self, system_update_id):
        """Retrieve System Update Information.

        :param string system_update_id: ID of system_update
        :return: SystemUpdate
        """
        return self._get(_system_update.SystemUpdate, system_update_id)

    def plans(self, **params):
        """List Plans."""
        return list(self._list(_plan.Plan, paginated=False, **params))

    def get_plan(self, plan_id):
        """Retrieve Plan Information.

        :param string plan_id: ID of plan
        :return: Plan
        """
        return self._get(_plan.Plan, plan_id)

    def health_monitors(self, **params):
        """List Health Monitors."""
        return list(self._list(
            _health_monitor.HealthMonitor, paginated=False, **params))

    def create_health_monitor(self, port, protocol, load_balancer_id,
                              name=None, description=None, tags=None,
                              interval=None, retry=None, timeout=None,
                              path=None, http_status_code=None
                              ):
        """Create Health Monitor.

        :param string port: Port of Health Monitor
        :param string protocol: Protocol of Health Monitor
        :param string load_balancer_id: Load Balancer ID
        :param string name: Name of Health Monitor
        :param string description: Description of Health Monitor
        :param dict tags: Tags of Health Monitor
        :param int interval: Interval of Health Monitor
        :param int retry: Retry count of Health Monitor
        :param int timeout: Timeout of Health Monitor
        :param string path: Path of Health Monitor
        :param string http_status_code: HTTP Status code of Health Monitor

        :return: Health Monitor
        """
        body = {
            'port': port,
            'protocol': protocol,
            'load_balancer_id': load_balancer_id
        }
        if name is not None:
            body["name"] = name
        if description is not None:
            body["description"] = description
        if tags is not None:
            body["tags"] = tags
        if interval is not None:
            body["interval"] = interval
        if retry is not None:
            body["retry"] = retry
        if timeout is not None:
            body["timeout"] = timeout
        if path is not None:
            body["path"] = path
        if http_status_code is not None:
            body["http_status_code"] = http_status_code
        return self._create(_health_monitor.HealthMonitor, **body)

    def get_health_monitor(self, health_monitor_id, changes=None):
        """Retrieve Health Monitor Information.

        :param string health_monitor_id: ID of Health Monitor
        :param bool changes: Whether or not to retrieve staged configuration
        :return: Health Monitor
        """
        health_monitor = _health_monitor.HealthMonitor()
        return health_monitor.get_resource(self.session, health_monitor_id,
                                           changes)

    def update_health_monitor(self, health_monitor_id, **params):
        """Update Health Monitor Attributes.

        :param string health_monitor_id: ID of Health Monitor
        :attrs **params: Parameters for health monitor update.

            * string name: Name of Health Monitor
            * string description: Description of Health Monitor
            * dict tags: Tags of Health Monitor

        :return: Health Monitor
        """
        return self._update(_health_monitor.HealthMonitor, health_monitor_id,
                            **params)

    def delete_health_monitor(self, health_monitor_id, ignore_missing=False):
        """Delete Health Monitor.

        :param string health_monitor_id: ID of Health Monitor
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent health monitor.
        :return: None
        """
        self._delete(_health_monitor.HealthMonitor, health_monitor_id,
                     ignore_missing=ignore_missing)

    def create_staged_health_monitor_configuration(
            self, health_monitor_id,
            port=None, protocol=None, interval=None, retry=None,
            timeout=None,
            path=None, http_status_code=None
    ):
        """Create Staged Health Monitor Configuration.

        :param string health_monitor_id: ID of Health Monitor
        :param int port: Port of Health Monitor
        :param string protocol: Protocol of Health Monitor
        :param int interval: Interval of Health Monitor
        :param int retry: Retry count of Health Monitor
        :param int timeout: Timeout of Health Monitor
        :param string path: Path of Health Monitor
        :param string http_status_code: HTTP Status code of Health Monitor
        :return: Health Monitor
        """
        body = {}
        if port is not None:
            body["port"] = port
        if protocol is not None:
            body["protocol"] = protocol
        if interval is not None:
            body["interval"] = interval
        if retry is not None:
            body["retry"] = retry
        if timeout is not None:
            body["timeout"] = timeout
        if path is not None:
            body["path"] = path
        if http_status_code is not None:
            body["http_status_code"] = http_status_code

        health_monitor = _health_monitor.HealthMonitor()
        return health_monitor.create_staged_configuration(self.session,
                                                          health_monitor_id,
                                                          **body)

    def get_staged_health_monitor_configuration(self, health_monitor_id):
        """Retrieve Staged Health Monitor Configuration.

        :param string health_monitor_id: ID of Health_monitor
        :return: Health Monitor
        """
        health_monitor = _health_monitor.HealthMonitor()
        return health_monitor.get_staged_configuration_resource(self.session, health_monitor_id)

    def update_staged_health_monitor_configuration(
            self, health_monitor_id,
            port=None, protocol=None, interval=None, retry=None,
            timeout=None,
            path=None, http_status_code=None
    ):
        """Update Staged Health Monitor Configuration.

        :param string health_monitor_id: ID of Health Monitor
        :param int port: Port of Health Monitor
        :param string protocol: Protocol of Health Monitor
        :param int interval: Interval of Health Monitor
        :param int retry: Retry count of Health Monitor
        :param int timeout: Timeout of Health Monitor
        :param string path: Path of Health Monitor
        :param string http_status_code: HTTP Status code of Health Monitor
        :return: Health Monitor
        """
        body = {}
        if port is not None:
            body["port"] = port
        if protocol is not None:
            body["protocol"] = protocol
        if interval is not None:
            body["interval"] = interval
        if retry is not None:
            body["retry"] = retry
        if timeout is not None:
            body["timeout"] = timeout
        if path is not None:
            body["path"] = path
        if http_status_code is not None:
            body["http_status_code"] = http_status_code

        health_monitor = _health_monitor.HealthMonitor()
        return health_monitor.update_staged_configuration(self.session,
                                                          health_monitor_id,
                                                          **body)

    def cancel_staged_health_monitor_configuration(self, health_monitor_id):
        """Delete Staged Health Monitor Configuration.

        :param string health_monitor_id: ID of Health Monitor
        :return: None
        """
        health_monitor = _health_monitor.HealthMonitor()
        health_monitor.cancel_staged_configuration(self.session,
                                                   health_monitor_id)

    def policies(self, **params):
        """List Policies."""
        return list(self._list(_policy.Policy, paginated=False, **params))

    def create_policy(self, health_monitor_id, listener_id,
                      default_target_group_id, load_balancer_id,
                      name=None, description=None, tags=None, algorithm=None,
                      persistence=None, idle_timeout=None, sorry_page_url=None,
                      source_nat=None, certificate_id=None, tls_policy_id=None
                      ):
        """Create Policy.

        :param string health_monitor_id: Health Monitor ID of Policy
        :param string listener_id: Listener ID of Policy
        :param string default_target_group_id: Default Target Group ID
        :param string load_balancer_id: Load Balancer ID
        :param string name: Name of Policy
        :param string description: Description of Policy
        :param dict tags: Tags of Policy
        :param string algorithm: Algorithm of Policy
        :param string persistence: Persistence of Policy
        :param integer idle_timeout: Idle Timeout of Policy
        :param string sorry_page_url: Sorry page URL
        :param string source_nat: Source NAT
        :param string certificate_id: Certificate ID
        :param string tls_policy_id: TLS Policy ID
        :return: Policy
        """
        body = {
            "health_monitor_id": health_monitor_id,
            "listener_id": listener_id,
            "default_target_group_id": default_target_group_id,
            "load_balancer_id": load_balancer_id,
        }
        if name is not None:
            body["name"] = name
        if description is not None:
            body["description"] = description
        if tags is not None:
            body["tags"] = tags
        if algorithm is not None:
            body["algorithm"] = algorithm
        if persistence is not None:
            body["persistence"] = persistence
        if idle_timeout is not None:
            body["idle_timeout"] = idle_timeout
        if sorry_page_url is not None:
            body["sorry_page_url"] = sorry_page_url
        if source_nat is not None:
            body["source_nat"] = source_nat
        if certificate_id is not None:
            body["certificate_id"] = certificate_id
        if tls_policy_id is not None:
            body["tls_policy_id"] = tls_policy_id
        return self._create(_policy.Policy, **body)

    def get_policy(self, policy_id, changes=None):
        """Retrieve Policy Information.

        :param string policy_id: ID of Policy
        :param bool changes: Whether or not to retrieve staged configuration
        :return: Policy
        """
        policy = _policy.Policy()
        return policy.get_resource(self.session, policy_id, changes)

    def update_policy(self, policy_id, **params):
        """Update Policy Attributes.

        :param string policy_id: ID of Policy
        :attrs **params: Parameters for policy update.

            * string name: Name of Policy
            * string description: Description of Policy
            * dict tags: Tags of Policy

        :return: Policy
        """
        return self._update(_policy.Policy, policy_id, **params)

    def delete_policy(self, policy_id, ignore_missing=False):
        """Delete Policy.

        :param string policy_id: ID of Policy
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent policy.
        :return: None
        """
        self._delete(_policy.Policy, policy_id, ignore_missing=ignore_missing)

    def create_staged_policy_configuration(self, policy_id,
                                           algorithm=None, persistence=None,
                                           idle_timeout=None, sorry_page_url=None,
                                           source_nat=None, certificate_id=None,
                                           health_monitor_id=None,
                                           listener_id=None,
                                           default_target_group_id=None,
                                           tls_policy_id=None
                                           ):
        """Create Staged Policy Configuration.

        :param string policy_id: ID of Policy
        :param string algorithm: Algorithm of Policy
        :param string persistence: Persistence of Policy
        :param integer idle_timeout: Idle Timeout of Policy
        :param string sorry_page_url: Sorry page URL
        :param string source_nat: Source NAT
        :param string certificate_id: Certificate ID
        :param string health_monitor_id: Health Monitor ID of Policy
        :param string listener_id: Listener ID of Policy
        :param string default_target_group_id: Default Target Group ID
        :param string tls_policy_id: TLS Policy ID
        :return: Policy
        """
        body = {}
        if algorithm is not None:
            body["algorithm"] = algorithm
        if persistence is not None:
            body["persistence"] = persistence
        if idle_timeout is not None:
            body["idle_timeout"] = idle_timeout
        if sorry_page_url is not None:
            body["sorry_page_url"] = sorry_page_url
        if source_nat is not None:
            body["source_nat"] = source_nat
        if certificate_id is not None:
            body["certificate_id"] = certificate_id
        if health_monitor_id is not None:
            body["health_monitor_id"] = health_monitor_id
        if listener_id is not None:
            body["listener_id"] = listener_id
        if default_target_group_id is not None:
            body["default_target_group_id"] = default_target_group_id
        if tls_policy_id is not None:
            body["tls_policy_id"] = tls_policy_id

        policy = _policy.Policy()
        return policy.create_staged_configuration(self.session,
                                                  policy_id, **body)

    def get_staged_policy_configuration(self, policy_id):
        """Retrieve Staged Policy Configuration.

        :param string policy_id: ID of Policy
        :return: Policy
        """
        policy = _policy.Policy()
        return policy.get_staged_configuration_resource(self.session, policy_id)

    def update_staged_policy_configuration(self, policy_id,
                                           algorithm=None, persistence=None,
                                           idle_timeout=None, sorry_page_url=None,
                                           source_nat=None, certificate_id=None,
                                           health_monitor_id=None,
                                           listener_id=None,
                                           default_target_group_id=None,
                                           tls_policy_id=None
                                           ):
        """Update Staged Policy Configuration.

        :param string policy_id: ID of Policy
        :param string algorithm: Algorithm of Policy
        :param string persistence: Persistence of Policy
        :param integer idle_timeout: Idle Timeout of Policy
        :param string sorry_page_url: Sorry page URL
        :param string source_nat: Source NAT
        :param string certificate_id: Certificate ID
        :param string health_monitor_id: Health Monitor ID of Policy
        :param string listener_id: Listener ID of Policy
        :param string default_target_group_id: Default Target Group ID
        :param string tls_policy_id: TLS Policy ID
        :return: Policy
        """
        body = {}
        if algorithm is not None:
            body["algorithm"] = algorithm
        if persistence is not None:
            body["persistence"] = persistence
        if idle_timeout is not None:
            body["idle_timeout"] = idle_timeout
        if sorry_page_url is not None:
            body["sorry_page_url"] = sorry_page_url
        if source_nat is not None:
            body["source_nat"] = source_nat
        if certificate_id is not None:
            body["certificate_id"] = certificate_id
        if health_monitor_id is not None:
            body["health_monitor_id"] = health_monitor_id
        if listener_id is not None:
            body["listener_id"] = listener_id
        if default_target_group_id is not None:
            body["default_target_group_id"] = default_target_group_id
        if tls_policy_id is not None:
            body["tls_policy_id"] = tls_policy_id

        policy = _policy.Policy()
        return policy.update_staged_configuration(self.session,
                                                  policy_id, **body)

    def cancel_staged_policy_configuration(self, policy_id):
        """Delete Staged Policy Configuration.

        :param string policy_id: ID of Policy
        :return: None
        """
        policy = _policy.Policy()
        policy.cancel_staged_configuration(self.session, policy_id)

    def routes(self, **params):
        """List Routes."""
        return list(self._list(_route.Route, paginated=False, **params))

    def create_route(self, destination_cidr,
                     next_hop_ip_address, load_balancer_id,
                     name=None, description=None, tags=None):
        """Create Route.

        :param string destination_cidr: Destination CIDR of Route
        :param string next_hop_ip_address: Next Hop IP Address of Route
        :param string load_balancer_id: Load Balancer ID of Route
        :param string name: Name of Route
        :param string description: Description of Route
        :param dict tags: Tags of Route

        :return: Route
        """
        body = {
            'destination_cidr': destination_cidr,
            'next_hop_ip_address': next_hop_ip_address,
            'load_balancer_id': load_balancer_id
        }
        if name is not None:
            body["name"] = name
        if description is not None:
            body["description"] = description
        if tags is not None:
            body["tags"] = tags
        return self._create(_route.Route, **body)

    def get_route(self, route_id, changes=None):
        """Retrieve Route Information.

        :param string route_id: ID of Route
        :param bool changes: Whether or not to retrieve staged configuration
        :return: Route
        """
        route = _route.Route()
        return route.get_resource(self.session, route_id, changes)

    def update_route(self, route_id, **params):
        """Update Route Attributes.

        :param string route_id: ID of Route
        :attrs **params: Parameters for policy update.

            * string name: Name of Route
            * string description: Description of Route
            * dict tags: Tags of Route

        :return: Route
        """
        return self._update(_route.Route, route_id, **params)

    def delete_route(self, route_id, ignore_missing=False):
        """Delete Route.

        :param string route_id: ID of Route
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent route.
        :return: None
        """
        self._delete(_route.Route, route_id, ignore_missing=ignore_missing)

    def create_staged_route_configuration(
            self, route_id, next_hop_ip_address=None):
        """Create Staged Route Configuration.

        :param string route_id: ID of Route
        :param string next_hop_ip_address: Next Hop IP Address of Route
        :return: Route
        """
        body = {}
        if next_hop_ip_address is not None:
            body["next_hop_ip_address"] = next_hop_ip_address

        route = _route.Route()
        return route.create_staged_configuration(self.session,
                                                 route_id, **body)

    def get_staged_route_configuration(self, route_id):
        """Retrieve Staged Route Configuration.

        :param string route_id: ID of Route
        :return: Route
        """
        route = _route.Route()
        return route.get_staged_configuration_resource(self.session, route_id)

    def update_staged_route_configuration(
            self, route_id, next_hop_ip_address=None):
        """Update Staged Route Configuration.

        :param string route_id: ID of Route
        :param string next_hop_ip_address: Next Hop IP Address of Route
        :return: Route
        """
        body = {}
        if next_hop_ip_address is not None:
            body["next_hop_ip_address"] = next_hop_ip_address

        route = _route.Route()
        return route.update_staged_configuration(self.session,
                                                 route_id, **body)

    def cancel_staged_route_configuration(self, route_id):
        """Delete Staged Route Configuration.

        :param string route_id: ID of Route
        :return: None
        """
        route = _route.Route()
        route.cancel_staged_configuration(self.session, route_id)

    def rules(self, **params):
        """List Rules."""
        return list(self._list(_rule.Rule, paginated=False, **params))

    def create_rule(self, priority, target_group_id, policy_id, conditions,
                    name=None, description=None, tags=None):
        """Create Rule.

        :param string priority: Priority of Rule
        :param string target_group_id: Target group ID of Rule
        :param string policy_id: Policy ID of Rule
        :param string conditions: Condition IDs of Rule
        :param string name: Name of Rule
        :param string description: Description of Rule
        :param dict tags: Tags of Rule
        :return: Rule
        """
        body = {
            'priority': priority,
            'target_group_id': target_group_id,
            'policy_id': policy_id,
            'conditions': conditions
        }
        if name is not None:
            body["name"] = name
        if description is not None:
            body["description"] = description
        if tags is not None:
            body["tags"] = tags
        return self._create(_rule.Rule, **body)

    def get_rule(self, rule_id, changes=None):
        """Retrieve Rule Information.

        :param string rule_id: ID of Rule
        :param bool changes: Whether or not to retrieve staged configuration
        :return: Rule
        """
        rule = _rule.Rule()
        return rule.get_resource(self.session, rule_id, changes)

    def update_rule(self, rule_id, name=None, description=None, tags=None):
        """Update Rule Attributes.

        :param string rule_id: ID of Rule
        :param string name: Name of Rule
        :param string description: Description of Rule
        :param dict tags: Tags of Rule
        :return: Rule
        """
        body = {}
        if name is not None:
            body["name"] = name
        if description is not None:
            body["description"] = description
        if tags is not None:
            body["tags"] = tags
        return self._update(_rule.Rule, rule_id, **body)

    def delete_rule(self, rule_id, ignore_missing=False):
        """Delete Rule.

        :param string rule_id: ID of Rule
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent rule.
        :return: None
        """
        self._delete(_rule.Rule, rule_id, ignore_missing=ignore_missing)

    def create_staged_rule_configuration(
            self, rule_id,
            priority=None, target_group_id=None, conditions=None):
        """Create Staged Rule Configuration.

        :param string rule_id: ID of Rule
        :param string priority: Priority of Rule
        :param string target_group_id: Target Group ID of Rule
        :param string conditions: Conditions of Rule
        :return: Rule
        """
        body = {}
        if priority is not None:
            body["priority"] = priority
        if target_group_id is not None:
            body["target_group_id"] = target_group_id
        if conditions is not None:
            body["conditions"] = conditions

        rule = _rule.Rule()
        return rule.create_staged_configuration(self.session, rule_id,  **body)

    def get_staged_rule_configuration(self, rule_id):
        """Retrieve Staged Rule Configuration.

        :param string rule_id: ID of Rule
        :return: Rule
        """
        rule = _rule.Rule()
        return rule.get_staged_configuration_resource(self.session, rule_id)

    def update_staged_rule_configuration(
            self, rule_id,
            priority=None, target_group_id=None, conditions=None):
        """Create Staged Rule Configuration.

        :param string rule_id: ID of Rule
        :param string priority: Priority of Rule
        :param string target_group_id: Target Group ID of Rule
        :param string conditions: Conditions of Rule
        :return: Rule
        """
        body = {}
        if priority is not None:
            body["priority"] = priority
        if target_group_id is not None:
            body["target_group_id"] = target_group_id
        if conditions is not None:
            body["conditions"] = conditions

        rule = _rule.Rule()
        return rule.update_staged_configuration(self.session, rule_id, **body)

    def cancel_staged_rule_configuration(self, rule_id):
        """Delete Staged Rule Configuration.

        :param string rule_id: ID of Rule
        :return: None
        """
        rule = _rule.Rule()
        rule.cancel_staged_configuration(self.session, rule_id)

    def operations(self, **params):
        """List operations."""
        return list(self._list(
            _operation.Operation, paginated=False, **params))

    def get_operation(self, operation_id):
        """Retrieve operation.

        :param string operation_id: ID of specified operation.
        :return: Operation
        """
        return self._get(_operation.Operation, operation_id)

    def tls_policies(self, **params):
        """List TLS Policies."""
        return list(self._list(
            _tls_policy.TLSPolicy, paginated=False, **params))

    def get_tls_policy(self, tls_policy_id):
        """Retrieve TLS Policy Information.

        :param string tls_policy_id: ID of TLS Policy
        :return: TLS Policy
        """
        return self._get(
            _tls_policy.TLSPolicy, tls_policy_id)



from . import base

from ecl import resource2
from ecl.mvna import mvna_service


class TargetGroup(base.MVNABaseResource):
    resource_key = "target_group"
    resources_key = "target_groups"
    service = mvna_service.MVNAService("v1.0")
    base_path = '/' + service.version + '/target_groups'

    _query_mapping = base.MVNAQueryParameters(
        "id",
        "name",
        "description",
        "configuration_status",
        "operation_status",
        "load_balancer_id",
        "tenant_id"
    )

    # Capabilities
    allow_list = True
    allow_get = True
    allow_create = True
    allow_update = True
    allow_delete = True
    patch_update = True

    # Properties
    #: It identifies connection resource uniquely
    id = resource2.Body('id')
    #: Name of target group
    name = resource2.Body('name')
    #: Description of target group
    description = resource2.Body('description')
    #: Tags of target group
    tags = resource2.Body('tags')
    #: Configuration status of target group
    configuration_status = resource2.Body('configuration_status')
    #: Operation status of target group
    operation_status = resource2.Body('operation_status')
    #: Load balancer ID of target group
    load_balancer_id = resource2.Body('load_balancer_id')
    #: Tenant ID of target group
    tenant_id = resource2.Body('tenant_id')
    #: Members of target group
    members = resource2.Body('members')
    #: Current of target group
    current = resource2.Body('current')
    #: Staged of target group
    staged = resource2.Body('staged')



from . import base

from ecl import resource2
from ecl.mvna import mvna_service


class Certificate(resource2.Resource):
    resource_key = "certificate"
    resources_key = "certificates"
    service = mvna_service.MVNAService("v1.0")
    base_path = '/' + service.version + '/certificates'

    _query_mapping = base.MVNAQueryParameters(
        "id",
        "name",
        "description",
        "tenant_id",
        "ca_cert_status",
        "ssl_key_status",
        "ssl_cert_status"
    )

    # Capabilities
    allow_list = True
    allow_get = True
    allow_create = True
    allow_update = True
    allow_delete = True
    patch_update = True

    # Properties
    #: It identifies connection resource uniquely
    id = resource2.Body('id')
    #: Name of certificate
    name = resource2.Body('name')
    #: Description of certificate
    description = resource2.Body('description')
    #: Tags of certificate
    tags = resource2.Body('tags')
    #: Tenant ID
    tenant_id = resource2.Body('tenant_id')
    #: SSL Key
    ssl_key = resource2.Body('ssl_key')
    #: SSL Cert
    ssl_cert = resource2.Body('ssl_cert')
    #: Ca Cert
    ca_cert = resource2.Body('ca_cert')

    def upload(self, session, resource_id, **body):
        uri = self.base_path + '/%s/files' % resource_id
        resp = session.post(uri, endpoint_filter=self.service, json=body)
        self._translate_response(resp, has_body=False)
        return self



# -*- coding: utf-8 -*-
from . import base

from ecl import resource2
from ecl.mvna import mvna_service

class Operation(resource2.Resource):
    resource_key = "operation"
    resources_key = "operations"
    service = mvna_service.MVNAService("v1.0")
    base_path = '/' + service.version + '/operations'

    _query_mapping = base.MVNAQueryParameters(
        "id",
        "resource_id",
        "resource_type",
        "request_id",
        "request_type",
        "status",
        "tenant_id",
        "no_deleted",
        "latest"
    )

    # Capabilities
    allow_list = True
    allow_get = True

    # Properties
    #: ID of network appliance's operation
    id = resource2.Body('id')
    #: Target load balancer id of operation.
    resource_id = resource2.Body('resource_id')
    #: resource type of operation.
    resource_type = resource2.Body('resource_type')
    #: Request ID of operation.
    request_id = resource2.Body('request_id')
    #: A string representation of the request type.
    request_types = resource2.Body('request_types')
    #: Request body(JSON String) of operation.
    request_body = resource2.Body('request_body')
    #: Status of operation.
    status = resource2.Body('status')
    #: Reception datetime of operation.
    reception_datetime = resource2.Body('reception_datetime')
    #: Commit datetime of operation.
    commit_datetime = resource2.Body('commit_datetime')
    #: Warning of operation.
    warning = resource2.Body('warning')
    #: Error of operation.
    error = resource2.Body('error')
    #: Tenant ID of operation.
    tenant_id = resource2.Body('tenant_id')
